require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/rest/paging_criteria'
require 'uu/fairpay2/financialtransaction/bank_transaction_create'
require 'uu/fairpay2/financialtransaction/bank_transaction_get_attributes'
require 'uu/fairpay2/financialtransaction/bank_transaction_get_transaction_list'

# Module Unicorn Universe.
module UU
  # module Fairpay2.
  module Fairpay2
    # Module Financial Transaction.
    module FinancialTransaction
      # Module Bank Transaction.
      # This service allows managing bank transactions, withdrawals and deposits at Client Account Business Case(CABC).
      module BankTransaction
        extend self

        # Service path
        PATH = 'uu/fairpay2/financialtransaction/BankTransaction'
        # Creates new Bank Transaction. The command creates a new Bank Transaction on specified Business Case with passed DTO parameters.
        #
        # <b>Financial transaction types:</b><br>
        # bank transaction, deposit, withdrawal.
        # Transaction type is specified by <code>meta_artifact_uri</code> attribute.
        #
        # <b>Required attributes:</b><br>
        #   name, meta_artifact_uri, competent_role_uri, location_uri, amount, debit_business_case_uri, currency, credit_account_currency, debit_account_currency, exchange_rate_for_opp_side, amount_for_opp_side
        #
        # @param [UU::OS::UESURI] credit_business_case_uri UESURI of the Client Account Business Case.
        # @param [UU::Fairpay2::FinancialTransaction::BankTransactionCreate] bank_transaction DTO containing attributes of the new Financial Transaction.
        #
        # @return [UU::OS::UESURI] UESURI of the new Bank Transaction
        #
        #
        # @example
        #      # Prepare Client Account Business Case URI, where we want to create Bank Transaction.
        #      credit_business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:CLIENT_ACCOUNT_BUSINESS_CASE');
        #
        #      # Create Bank Transaction
        #      UU::Fairpay2::FinancialTransaction::BankTransaction.create(credit_business_case_uri,
        #          :name => "NAME OF BANK TRANSACTION",
        #          :competent_role_uri => "ues:TERRITORY:ROLE",
        #          :meta_artifact_uri => "ues:TERRITORY:META_ARTIFACT_OF_BANK_TRANSACTION",
        #          :location_uri => "ues:TERRITORY:FOLDER_OR_ORG_UNIT",
        #          :amount => "1000",
        #          :constant_symbol => "111",
        #          :debit_business_case_uri => "ues:TERRITORY:WITHDRAWAL_CLIENT_ACCOUNT_BUSINESS_CASE",
        #          :specific_symbol => "222",
        #          :message_payer => "message",
        #          :message_recipient => "message",
        #          :credit_account_number => "12345/123",
        #          :debit_account_number => "54321/123",
        #          :currency => "CZK",
        #          :credit_account_currency => "CZK",
        #          :debit_account_currency => "CZK");
        #
        #      # Prepare Client Account Business Case URI, where we want to create Bank Transaction.
        #      # Attribute :credit_account_number must be authorized account.
        #      credit_business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:BUSINESS_CASE_FOR_WITHDRAWALS');
        #      # Create Withdrawal
        #      UU::Fairpay2::FinancialTransaction::BankTransaction.create(credit_business_case_uri,
        #          :name => "NAME OF WITHDRAWAL",
        #          :competent_role_uri => "ues:TERRITORY:ROLE",
        #          :meta_artifact_uri => "ues:TERRITORY:META_ARTIFACT_OF_WITHDRAWAL",
        #          :location_uri => "ues:TERRITORY:FOLDER_OR_ORG_UNIT",
        #          :amount => "1000",
        #          :constant_symbol => "111",
        #          :debit_business_case_uri => "ues:TERRITORY:DEPOSIT_CLIENT_ACCOUNT_BUSINESS_CASE",
        #          :specific_symbol => "222",
        #          :message_payer => "message",
        #          :message_recipient => "message",
        #          :credit_account_number => "12345/123",
        #          :debit_account_number => "54321/123",
        #          :currency => "CZK",
        #          :credit_account_currency => "CZK",
        #          :debit_account_currency => "CZK");
        #
        #      # Prepare Client Account Business Case URI, where we want to create Bank Transaction.
        #      # Attribute :debit_account_number must be authorized account.
        #      credit_business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:DEPOSIT_CLIENT_ACCOUNT_BUSINESS_CASE');
        #      # Create Deposit
        #      UU::Fairpay2::FinancialTransaction::BankTransaction.create(credit_business_case_uri,
        #          :name => "NAME OF DEPOSIT",
        #          :competent_role_uri => "ues:TERRITORY:ROLE",
        #          :meta_artifact_uri => "ues:TERRITORY:META_ARTIFACT_OF_DEPOSIT",
        #          :location_uri => "ues:TERRITORY:FOLDER_OR_ORG_UNIT",
        #          :amount => "1000",
        #          :constant_symbol => "111",
        #          :currency => "CZK",
        #          :debit_business_case_uri => "ues:TERRITORY:BUSINESS_CASE_FOR_DEPOSITS",
        #          :variable_symbol => "222",
        #          :specific_symbol => "222",
        #          :message_payer => "message",
        #          :message_recipient => "message",
        #          :credit_account_number => "12345/123",
        #          :debit_account_number => "54321/123",
        #          :currency => "CZK",
        #          :credit_account_currency => "CZK",
        #          :debit_account_currency => "CZK");
        def create(credit_business_case_uri, bank_transaction = nil)
          svc = UU::OS::REST::RemoteClient.new(BankTransaction)
          bank_transaction_create = UU::Fairpay2::FinancialTransaction::BankTransactionCreate.new(bank_transaction).to_json

          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('create', credit_business_case_uri, bank_transaction_create)
            return UU::OS::UESURI.new(res)
          end
        end

        # Set selected attributes to existing Bank Transaction with passed DTO parameters.
        #
        # @param [UU::OS::UESURI] bank_transaction_uri UESURI of the existing Bank Transaction.
        # @param [UU::Fairpay2::FinancialTransaction::BankTransactionSetAttributes] bank_transaction DTO containing attributes to modify Financial Transaction.
        #
        # @return [UU::OS::UESURI] UESURI of the modified Bank Transaction
        #
        #
        # @example
        #      # Prepare Bank Transaction URI, where we want to modify attributes.
        #      bank_transaction_uri = UU::OS::UESURI.new('ues:TERRITORY:BANK_TRANSACTION');
        #
        #      # Sets Bank Transaction
        #      UU::Fairpay2::FinancialTransaction::BankTransaction.set_attributes(bank_transaction_uri,
        #          :message_payer => "new message");
        #
        def set_attributes(bank_transaction_uri, bank_transaction = nil)
          svc = UU::OS::REST::RemoteClient.new(BankTransaction)
          bank_transaction_set = UU::Fairpay2::FinancialTransaction::BankTransactionSetAttributes.new(bank_transaction).to_json
          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('setAttributes', bank_transaction_uri, bank_transaction_set)
            return UU::OS::UESURI.new(res)
          end
        end

        # Obtains Bank Transaction attributes from REST service and returns them
        #
        # @param [String,UU::OS::UESURI] bank_transaction_uri UESURI of the Bank Transaction.
        #
        # @return UU::Fairpay2::FinancialTransaction::BankTransactionAttributes] Bank Transaction attributes
        #
        #
        # @example
        #      # Prepare Financial Transaction URI, where we want to get attributes.
        #      bank_transaction_uri = UU::OS::UESURI.new('ues:TERRITORY:FINANCIAL_TRANSACTION');
        #
        #      # Get Financial Transation attributes
        #      UU::Fairpay2::FinancialTransaction::BankTransaction.get_attributes(bank_transaction_uri);
        def get_attributes(bank_transaction_uri)
          svc = UU::OS::REST::RemoteClient.new(BankTransaction)
          UU::OS::QoS::QoSHandler.auto_retry do
            attrs = svc.get('getAttributes', bank_transaction_uri)
            return UU::Fairpay2::FinancialTransaction::BankTransactionAttributes.new(attrs)
          end
        end

        # Retrieves a result list of financial transaction for specified client account business case by UESURI.
        # The resulting attachment list is filtered according to the criteria provided by the criteria attribute.
        #
        # @param [UU::OS::UESURI] client_business_case_uri UESURI of the client account business case.
        # @param [UU::Fairpay2::FinancialTransaction::BankTransactionGetTransactionList] criteria DTO containing paging and filtering criteria. Available parameters are: {BankTransactionAttributes#creation_time creationTime}, {BankTransactionAttributes#meta_artifact_uri metaArtifactUri}.
        # @option criteria [String] :query The UES query string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there.
        #
        # @return [UU::OS::REST::ResultList<UU::Fairpay2::FinancialTransaction::BankTransactionGetTransactionList, UU::Fairpay2::FinancialTransaction::BankTransactionAttributes>] List of financial transaction.
        #
        #
        # @example
        #      # Prepare Client Account Business Case URI, where we want to get Financial Transaction list.
        #      client_business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:CLIENT_ACCOUNT_BUSINESS_CASE');
        #
        #      # Get transaction list on  specific Client Account Business Case
        #      UU::Fairpay2::FinancialTransaction::BankTransaction.get_transaction_list(client_business_case_uri,
        #       :query => "creationTime > '2015-01-01' AND metaArtifactUri = 'ues:TERRITORY:META_ARTIFACT_OF_FINANCIAL_TRANSACTION'");
        def get_transaction_list(client_business_case_uri, criteria = nil)
          svc = UU::OS::REST::RemoteClient.new(BankTransaction)
          dto = UU::Fairpay2::FinancialTransaction::BankTransactionGetTransactionList.new(criteria)
          svc.add_parameter('pageIndex', dto.page_index)
          svc.add_parameter('pageSize', dto.page_size)
          svc.add_parameter('query', dto.query)
          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.get('getTransactionList', client_business_case_uri)
            return UU::OS::REST::ResultList.new(UU::Fairpay2::FinancialTransaction::BankTransactionGetTransactionList, UU::Fairpay2::FinancialTransaction::BankTransactionAttributes, res)
          end
        end

      end
    end
  end
end
