require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/fairpay2/businesscase/client_account_business_case_create'
require 'uu/fairpay2/businesscase/client_account_business_case_attributes'
require 'uu/fairpay2/businesscase/client_account_business_case_set_attributes'

# Module Unicorn Universe.
module UU
  # module Fairpay2.
  module Fairpay2
    # Module Business Case.
    module BusinessCase
      # Module Client Account Business Case.
      # This service allows managing Client Account Business Case(CABC).
      module ClientAccountBusinessCase
        extend self

        # Service path
        PATH = 'uu/fairpay2/businesscase/ClientAccountBusinessCase'
        # Creates a new Client Account Business Case. The command creates a new Client Account Business Case
        # in a specified location  with passed DTO parameters.<br>
        #
        # <b>Required attributes:</b><br>
        #   name, meta_artifact_uri, user_universe_id, client_uuee_universe_id, client_account_number, available_balance, current_balance, authorized_persons, authorized_accounts, daily_limit, weekly_limit, monthly_limit, currency_code
        #
        #
        # @param [UU::OS::UESURI] location_uri UESURI of the folder  or organizational unit where Client Account Business Case will be created.
        # @param [UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseCreate] business_case DTO containing attributes of the new Client Account Business Case
        #
        # @return [UU::OS::UESURI] UESURI of the new Client Account Business Case
        #
        #
        # @example
        #      # Prepare folder or Organization Unit URI, where we want to create Client Account Business Case.
        #      location_uri = UU::OS::UESURI.new('ues:TERRITORY:ARTIFACT_FOLDER_OR_OU');
        #
        #      # Create personal CABC
        #        UU::Fairpay2::BusinessCase::ClientAccountBusinessCase.create(location_uri,
        #        :name => "1000000",
        #        :meta_artifact_uri => UU::OS::UESURI.new('ues:TERRITORY:META_ARTIFACT_CLIENT_ACCOUNT_BUSINESS_CASE'),
        #        :user_universe_id => "123-4",
        #        :client_uuee_universe_id => "436-1",
        #        :client_account_number => "123456/0100",
        #        :available_balance => "1000000",
        #        :current_balance => "1000000",
        #        :authorized_persons => "436-1",
        #        :authorized_accounts => "123456/0100",
        #        :daily_limit => "10000",
        #        :weekly_limit => "100000",
        #        :monthly_limit => "1000000",;
        #        :currency_code => "CZK");
        def create(location_uri, business_case = nil)
          svc = UU::OS::REST::RemoteClient.new(ClientAccountBusinessCase)
          clibc_create = UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseCreate.new(business_case).to_json

          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('create', location_uri, clibc_create)
            return UU::OS::UESURI.new(res)
          end
        end

        # Obtains Client Account Business Case attributes from REST service and returns them
        #
        # @param [String,UU::OS::UESURI] business_case_uri UESURI of the Client Account Business Case
        # @return [UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseAttributes] Client Account Business Case attributes
        #
        #
        # @example
        #      # Prepare Client Account Business Case URI, where we want to get attributes.
        #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:CLIENT_ACCOUNT_BUSINESS_CASE');
        #
        #      # Get attributes from CABC
        #        UU::Fairpay2::BusinessCase::ClientAccountBusinessCase.get_attributes(business_case_uri);
        def get_attributes(business_case_uri)
          svc = UU::OS::REST::RemoteClient.new(ClientAccountBusinessCase)

          UU::OS::QoS::QoSHandler.auto_retry do
            attrs = svc.post('getAttributes', business_case_uri)
            return UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseAttributes.new(attrs)
          end
        end

        # Sets attributes of Client Account Business Case.
        #
        # @param [String,UU::OS::UESURI] business_case_uri UESURI of modified Client Account Business Case
        # @param [UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseSetAttributes] business_case_attributes DTO containing modified attributes of the Client Account Business Case
        #
        # @return [UU::OS::UESURI] UESURI of modified Client Account Business Case
        #
        #
        # @example
        #      # Prepare Client Account Business Case URI, where we want to set attributes.
        #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:CLIENT_ACCOUNT_BUSINESS_CASE');
        #
        #      # Set selected attributes to CABC
        #        UU::Fairpay2::BusinessCase::ClientAccountBusinessCase.set_attributes(business_case_uri,
        #        :daily_limit => "1000000",
        #        :name => "new name");
        def set_attributes(business_case_uri, business_case_attributes = nil)
          svc = UU::OS::REST::RemoteClient.new(ClientAccountBusinessCase)
          bc_attrs = UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseSetAttributes.new(business_case_attributes).to_json

          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('setAttributes', business_case_uri, bc_attrs)
            return UU::OS::UESURI.new(res)
          end
        end

        # Obtains Client Account Business Case daily, weekly, monthly sums of outcomes transactions from REST service and returns them
        #
        # @deprecated Use {UU::Fairpay2::BusinessCase::ClientAccountBusinessCase#debit_transaction_sums} instead.
        # @param [String,UU::OS::UESURI] business_case_uri UESURI of the Client Account Business Case
        # @return [UU::FairPay2::BusinessCase::ClientAccountBusinessCaseSums] Client Account Business Case attributes
        #
        #
        # @example
        #      # Prepare Client Account Business Case URI, where we want to get sums.
        #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:CLIENT_ACCOUNT_BUSINESS_CASE');
        #
        #      # Get sums from CABC
        #        UU::Fairpay2::BusinessCase::ClientAccountBusinessCase.get_sums(business_case_uri);
        def get_sums(business_case_uri)
          svc = UU::OS::REST::RemoteClient.new(ClientAccountBusinessCase)

          UU::OS::QoS::QoSHandler.auto_retry do
            attrs = svc.post('debitTransactionSums', business_case_uri)
            return UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseSums.new(attrs)
          end
        end


        # Obtains Client Account Business Case daily, weekly, monthly sums of outgoing transactions from REST service and returns them
        #
        # @param [String,UU::OS::UESURI] business_case_uri UESURI of the Client Account Business Case
        # @return [UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseSums] Client Account Business Case attributes
        #
        #
        # @example
        #      # Prepare Client Account Business Case URI, where we want to get outgoing sums.
        #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:CLIENT_ACCOUNT_BUSINESS_CASE');
        #
        #      # Get sums from CABC
        #        UU::Fairpay2::BusinessCase::ClientAccountBusinessCase.debit_transaction_sums(business_case_uri);
        def debit_transaction_sums(business_case_uri)
          svc = UU::OS::REST::RemoteClient.new(ClientAccountBusinessCase)

          UU::OS::QoS::QoSHandler.auto_retry do
            attrs = svc.post('debitTransactionSums', business_case_uri)
            return UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseSums.new(attrs)
          end
        end

        # Returns UESURI of client account business case specified by client account number in value object
        #
        # @deprecated Use {UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseSearch#query} instead.
        # @param [String,UU::OS::UESURI] territory_uri UESURI of the territory where we want to search requested Client Account Business Case
        # @param [UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseAccountNumber] account_number Client Account Business Case Account Number attributes
        # @return [UU::OS::UESURI] UESURI of the founded Client Account Business Case
        #
        # <b>Required attribute:</b><br>
        #   client_account_number
        #
        # @example
        #      # Prepare territory URI, where we want to find client account business case.
        #      territory_uri = UU::OS::UESURI.new('ues:TERRITORY:TERRITORY');
        #
        #      # Find Client Account Business Case
        #        UU::Fairpay2::BusinessCase::ClientAccountBusinessCase.find_by_account_number(territory_uri,
        #        :client_account_number => "1000000");
        def find_by_account_number(territory_uri, account_number = nil)
          svc = UU::OS::REST::RemoteClient.new(ClientAccountBusinessCase)
          account_number_dto = UU::Fairpay2::BusinessCase::ClientAccountBusinessCaseAccountNumber.new(account_number).to_json

          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('findByAccountNumber', territory_uri, account_number_dto)
            return UU::OS::UESURI.new(res)
          end
        end

      end
    end
  end
end
